"""TA-Lib compatible APIs for AKQuant."""

from .funcs import (
    ABS,
    ACOS,
    AD,
    ADD,
    ADOSC,
    ADX,
    ADXR,
    APO,
    AROON,
    AROONOSC,
    ASIN,
    ATAN,
    ATR,
    AVGDEV,
    AVGPRICE,
    BBANDS,
    BETA,
    BOP,
    CCI,
    CEIL,
    CLAMP01,
    CLIP,
    CMO,
    CORREL,
    COS,
    COSH,
    COVAR,
    CUBE,
    DEMA,
    DIV,
    DX,
    EMA,
    EXP,
    FLOOR,
    HT_TRENDLINE,
    KAMA,
    LINEARREG,
    LINEARREG_ANGLE,
    LINEARREG_INTERCEPT,
    LINEARREG_R2,
    LINEARREG_SLOPE,
    LN,
    LOG10,
    MACD,
    MAMA,
    MAX,
    MAX2,
    MAXINDEX,
    MEDPRICE,
    MFI,
    MIDPOINT,
    MIDPRICE,
    MIN,
    MIN2,
    MININDEX,
    MINMAX,
    MINMAXINDEX,
    MINUS_DI,
    MOD,
    MOM,
    MULT,
    NATR,
    OBV,
    PLUS_DI,
    POW,
    PPO,
    RANGE,
    ROC,
    ROCP,
    ROCR,
    ROCR100,
    ROUND,
    RSI,
    SAR,
    SIGN,
    SIN,
    SINH,
    SMA,
    SQ,
    SQRT,
    STDDEV,
    STOCH,
    SUB,
    SUM,
    T3,
    TAN,
    TANH,
    TEMA,
    TRANGE,
    TRIMA,
    TRIX,
    TSF,
    TYPPRICE,
    ULTOSC,
    VAR,
    WCLPRICE,
    WILLR,
    WMA,
)

__all__ = [
    "AD",
    "ABS",
    "ACOS",
    "ADD",
    "ADOSC",
    "AROON",
    "AROONOSC",
    "DX",
    "BETA",
    "CORREL",
    "COVAR",
    "CUBE",
    "DIV",
    "LINEARREG",
    "LINEARREG_ANGLE",
    "LINEARREG_INTERCEPT",
    "LINEARREG_R2",
    "LINEARREG_SLOPE",
    "TSF",
    "HT_TRENDLINE",
    "APO",
    "ATAN",
    "ASIN",
    "AVGDEV",
    "CEIL",
    "CLAMP01",
    "COS",
    "COSH",
    "PPO",
    "POW",
    "PLUS_DI",
    "MINUS_DI",
    "MOD",
    "ULTOSC",
    "WMA",
    "T3",
    "MAMA",
    "LN",
    "MAX",
    "MAX2",
    "MAXINDEX",
    "MINMAXINDEX",
    "TRIMA",
    "MIDPOINT",
    "STDDEV",
    "SUM",
    "VAR",
    "ADXR",
    "CMO",
    "TRANGE",
    "RANGE",
    "SQRT",
    "MEDPRICE",
    "MIDPRICE",
    "AVGPRICE",
    "MIN",
    "MIN2",
    "MININDEX",
    "MINMAX",
    "TYPPRICE",
    "WCLPRICE",
    "BOP",
    "ROC",
    "ROCP",
    "ROCR",
    "ROCR100",
    "LOG10",
    "EXP",
    "ROUND",
    "CLIP",
    "SIGN",
    "SIN",
    "SINH",
    "SQ",
    "MOM",
    "OBV",
    "WILLR",
    "TRIX",
    "TEMA",
    "KAMA",
    "NATR",
    "SAR",
    "DEMA",
    "MFI",
    "CCI",
    "ADX",
    "STOCH",
    "RSI",
    "SMA",
    "EMA",
    "ATR",
    "MACD",
    "FLOOR",
    "BBANDS",
    "TAN",
    "TANH",
    "SUB",
    "MULT",
]
